      PROGRAM SOFA EARTH ORIENTATION EXAMPLES
*
* Demonstration program of SOFA Routines for Earth Orientation
*
* Examples to:
*  Form the celestial-to-intermediate matrix for a given date explicitly
*  using the series representation for X and Y (IAU 2006 precession,
*  IAU 2000A nutation), and the CIO locator s.
*
* Example 2 using series form for X & Y
* Example 3 equivalently using bias-precession-nutation angles for X & Y
*
* Catherine Hohenkerk
* Chair, IAU SOFA Board
* 2013 September 15
*
      IMPLICIT NONE
      INTEGER          IY, IM, ID, IHR, IMN, J
      DOUBLE PRECISION SEC, DX, DY
      DOUBLE PRECISION TT1, TT2, X1,Y1,S1, C1(3,3), X2,Y2,S2, C2(3,3)

*  Arc seconds to radians
      DOUBLE PRECISION DAS2R
      PARAMETER      ( DAS2R = 4.848136811095359935899141D-6 )

* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

      WRITE (6, 935)
      WRITE (6, '(A)')'2013 Journees Workshop, Earth Orientation using I
     :AU SOFA Routines'
      WRITE (6, 935)
      WRITE (6, *)

* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

* User parameters for both examples
* Civil date, TT
      IY = 2013
      IM = 9
      ID = 15
      IHR = 17
      IMN = 30
      SEC = 0D0

* IERS CIP offsets wrt IAU 2006/2000A (guess?)("->radians) 
      DX =  -0.2D-06 * DAS2R
      DY =  -0.10D-06 * DAS2R 

      CALL iau_DTF2D ( 'TT', IY, IM, ID, IHR, IMN, SEC, TT1, TT2, J )

* Print dates for both examples.
      WRITE (6, 935)
      WRITE (6,'(A)') 'Examples 2 & 3: GCRS->CIRS, using SOFA Routines
     : C2I'
      WRITE (6, 935)
      WRITE (6, 950) 'Date and time (TT)', IY, IM, ID, IHR, IMN, SEC

      WRITE (6, 930) 'JD (TT) = TT1 + TT2', TT1, TT2, TT1+TT2
      WRITE (6,'(/A)') 'IERS Parameters:'
      WRITE (6, 945) DX/DAS2R, DY/DAS2R

* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

      CALL C2I_EXAMPLE_2 ( TT1, TT2, DX, DY, X1, Y1, S1, C1 )

      CALL C2I_EXAMPLE_3 ( TT1, TT2, DX, DY, X2, Y2, S2, C2 )

* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

* Differences
      WRITE (6, 935)
      CALL DPXYS ( 'Differences using iau_XY06 and iau_XYS06A',
     :                                       X1-X2,  Y1-Y2,  S1-S2 )
      WRITE (6, 935)
      WRITE (6, 935)

* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

 930  FORMAT(/ 2X, A, ' =',F10.1,' + ' F12.10, ' = ', F18.10) 
 935  FORMAT('+ ',33('- '), '-+')
 945  FORMAT( 2X, '     DX (") = ', F13.9, 3X,'DY (") = ' F13.9)
 950  FORMAT(/A, 1X, I4,3(1X,I2.2),':',I2.2,':',F5.2)

*+----------------------------------------------------------------------

      END
      
      SUBROUTINE C2I_EXAMPLE_2 ( TT1, TT2, DX, DY, X, Y, S, C )
*+
* Example 2: SOFA Demonstration routine for the Journees 2013 Workshop
*
*  Form the celestial-to-intermediate matrix for a given date explicitly
*  using the series representation for X and Y (IAU 2006 precession,
*  IAU 2000A nutation), and the CIO locator s.
*
*  Given:
*     TT1,TT2    d       TT as a 2-part Julian Date
*     DX, DY     d       IERS CIP offsets wrt IAU 2006/2000A
*
*  Returned:
*     X,Y        d       Celestial Intermediate Pole
*     S          d       the CIO locator s
*     C          d(3,3)  celestial-to-intermediate matrix
*
*  Notes:
*
*  1) The TT date TT1+TT2 is a Julian Date, apportioned in any
*     convenient way between the two arguments (see SOFA routines).
*
*  2) DX, DY, X, Y. S are all in radians.
*
*-----------------------------------------------------------------------

      IMPLICIT NONE
      DOUBLE PRECISION TT1, TT2, DX, DY, X, Y, S, C(3,3)
      DOUBLE PRECISION iau_S06

* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

* Step 1: - Given the TT instant calculate the X, Y of the CIP
* using the series formulation (IAU 2006/2000A) for CIP, and
* the CIO locator s.

      CALL iau_XY06 ( TT1, TT2, X, Y )

      S = iau_S06 ( TT1, TT2, X, Y )

* Step 2: Incorporating the IERS corrections DX, DY calculate
* the matrix C, the GCRS to CIRS matrix.

      CALL iau_C2IXYS ( X+DX, Y+DY, S, C )
      
* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

      WRITE ( 6, 935)
 935  FORMAT('+ ',33('- '), '-+')
      WRITE(6, '(/A)') 'Example 2'      
      CALL PXYS ( 'X, Y, s using iau_XY06 and iau_S06', X, Y, S )
      CALL PXYS ( 'X+DX, Y+DY, s', X+DX, Y+DY, S )      
      CALL PMAT ( 'GCRS TO CIRS using iau_C2IXYS', C )

*+----------------------------------------------------------------------

      END

      SUBROUTINE C2I_EXAMPLE_3 ( TT1, TT2, DX, DY, X, Y, S, C )
*+
* Example 3:  SOFA Demonstration routine for the Journees 2013 Workshop
*
*  Form the celestial-to-intermediate matrix for a given date using
*  bias-precession-nutation angles (IAU 2006 precession and IAU 2000A
*  nutation) for X and Y, and the CIO locator s.
*
*  Given:
*     TT1,TT2    d       TT as a 2-part Julian Date
*
*  Returned:
*     DX, DY     d       IERS CIP offsets wrt IAU 2006/2000A
*     X,Y        d       Celestial Intermediate Pole
*     S          d       the CIO locator s
*     C          d(3,3)  celestial-to-intermediate matrix
*
*  Notes:
*
*  1) The TT date TT1+TT2 is a Julian Date, apportioned in any
*     convenient way between the two arguments (see SOFA routines).
*
*  2) DX, DY, X, Y. S are all in radians.
*
*-----------------------------------------------------------------------

      IMPLICIT NONE
      DOUBLE PRECISION TT1, TT2, DX, DY, X, Y, S, C(3,3)

* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

* Step 1: SOFA generated X, Y via IAU 2006/2000A bias-precession-
* nutation angles

      CALL iau_XYS06A ( TT1, TT2, X, Y, S )
      
* Step 2: Incorporating the IERS corrections DX, DY calculate
* the matrix C the GCRS to CIRS matrix.

      CALL iau_C2IXYS ( X+DX, Y+DY, S, C )

* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

      WRITE ( 6, 935)
 935  FORMAT('+ ',33('- '), '-+')
      WRITE(6, '(/A)') 'Example 3'      
      CALL PXYS ( 'X, Y, s using iau_XYS06A', X, Y, S )
      CALL PXYS ( 'X+DX, Y+DY, s', X+DX, Y+DY, S )
      CALL PMAT ( 'GCRS TO CIRS matrix using iau_C2IXYS', C )

*+----------------------------------------------------------------------

      END

      SUBROUTINE PMAT ( TEXT, R )

* Print the matrix R

      IMPLICIT NONE
      DOUBLE PRECISION R(3,3)
      INTEGER          I, J
      CHARACTER        TEXT*(*)

      WRITE (6,'(/A)') TEXT
      DO I = 1,3
       WRITE (6, 950) (R(I,J),J=1,3)
 950   FORMAT( 4X, 3F20.15)
      ENDDO

      END

      SUBROUTINE PXYS ( TEXT, X, Y, S )

* Print X, Y, s in arc seconds

      IMPLICIT NONE
      DOUBLE PRECISION X, Y, S
      CHARACTER        TEXT*(*)

*  Arc seconds to radians
      DOUBLE PRECISION DAS2R
      PARAMETER      ( DAS2R = 4.848136811095359935899141D-6 )

      WRITE (6, '(/A)') TEXT
      WRITE (6,950) 'X (") = ', X/DAS2R,
     :              'Y (") = ', Y/DAS2R,
     :              's (") = ', S/DAS2R
950   FORMAT(3X, A, F13.7)

      END

      SUBROUTINE DPXYS ( TEXT, DX, DY, DS )

* Print differences in X, Y, s in arc seconds

      IMPLICIT NONE
      DOUBLE PRECISION DX, DY, DS
      CHARACTER        TEXT*(*)

* Arc seconds to radians
      DOUBLE PRECISION DAS2R
      PARAMETER      ( DAS2R = 4.848136811095359935899141D-6 )

      WRITE (6, '(/A)') TEXT
      WRITE (6,950) 'X1 - X2 (") = ', DX/DAS2R,
     :              'Y1 - Y2 (") = ', DY/DAS2R,
     :              's1 - s2 (") = ', DS/DAS2R
950   FORMAT(2(4X, A, E9.2/), 4X, A, E9.2)

      END
