#include <stdio.h>
#include "sofa.h"
#include "sofam.h"

/* Arcseconds to radians */
#define DAS2R (4.848136811095359935899141e-6)

/*  Function prototypes. */
void Pmat ( char text[], double r[3][3] );
void Pxys ( char text[], double x, double y, double s );
void repjd ( char s[], double t1, double t2 );


int main()
/*
   PROGRAM SOFA EARTH ORIENTATION EXAMPLES
  
   Demonstration program of SOFA Routines for Earth Orientation
  
   This program does not follow any strict coding standards and
   is not part of the SOFA code.
  
   Taken from a version completed and debugged by John Bangert, 2013 Sep 10.
  
   Catherine Hohenkerk
   Chair, IAU SOFA Board
   2013 September 15
*/
{
   double  utc1, utc2, tai1, tai2, tt1, tt2, ut1a, ut1b;
   double sec, dx, dy, xp, yp, dut;

   double  x, y, s, c[3][3], era, eo, sp, w[3][3];
   double q0[3][3], q1[3][3];

   int iy, im, id, ihr, imn, j;

   printf ( "+ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -+\n" );
   printf ( "2013 Journees Workshop, Earth Orientation using IAU SOFA Routines\n" );
   printf ( "+ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -+\n\n" );
   
   printf ( "+ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -+\n" );
   printf ("Example 4: GCRS->TIRS & ITRS using SOFA ANSI C Routines \n"); 
   printf ( "+ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -+\n");

/* Civil date, UTC */ 
   iy = 2013;
   im = 9;
   id = 15;
   ihr = 17;
   imn = 30;
   sec = 0e0;

/* IERS CIP offsets wrt IAU 2006/2000A (radians) */
   dx = -0.2e-6 * DAS2R;  
   dy = -0.1e-6*  DAS2R;   

/* IERS polar motion, coordinates of CIP wrt ITRS (radians)  */
   xp = +0.1574e0 * DAS2R;   
   yp = +0.3076e0 * DAS2R;  

/* UT1-UTC (seconds)  */
   dut = +0.02792e0; 

   printf ( "\nDate and time (UTC) %4.4d %2.2d %2.2d %2.2d:%2.2d:%05.2f\n", 
     iy, im, id, ihr, imn, sec);  
   if ( iauDtf2d ( "UTC", iy, im, id, ihr, imn, sec, &utc1, &utc2 ) ) j= -1;

/* UTC -> TAI -> TT  */ 
   if ( iauUtctai ( utc1, utc2, &tai1, &tai2 ) ) j= -1;
   if ( iauTaitt  ( tai1, tai2, &tt1,  &tt2) ) j = -1;

/* UTC -> UT1 using IERS DUT = UT1-UTC */
   if ( iauUtcut1 ( utc1, utc2, dut, &ut1a, &ut1b) ) j = -1;
            
/* Output various instants  */
   repjd ( " JD (UTC) = ", utc1, utc2);
   repjd ( " JD (TAI) = ", tai1, tai2);
   repjd ( " JD (TT)  = ", tt1,  tt2);
   repjd ( " JD (UT1) = ", ut1a, ut1b);
   printf ("\n");

   printf ( "IERS Parameters:\n" ); 
   printf ( "       DX (\") = %13.9f   DY (\") = %13.9f\n",  dx/DAS2R, dy/DAS2R );              
   printf ( "       XP (\") = %10.6f      YP (\") = %10.6f\n",  xp/DAS2R, yp/DAS2R );      
   printf ( "  UT1-UTC (s) =   %.6f\n",  dut);  
   printf ("\n");

/* 
   Method A - Transformation matrix Q0 from GCRS to ITRS (C2T).
  
   A 1-call example using SOFA (IAU 2006/2000A) generated  
   quantities given the TT and UT1 instant.
*/
   iauC2t06a ( tt1, tt2, ut1a, ut1b, xp, yp, q0 );
   Pmat ( "GCRS TO ITRS matrix using iauC2t06a", q0 );
   printf ( "+ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -+\n\n" );

/*
   Method B - Transformation matrix from GCRS to TIRS & ITRS.                           
  
   GCRS to CIRS matrix (IAU 2006/2000A) using series formulation for  
   X, Y of CIP (Step 1 & 2 are equivalent to Example 2, alternatively 
   they could be replaced by Example 3)                              
    
   Step 1 - generate X, Y and s
*/
   iauXy06 ( tt1, tt2, &x, &y );
   s = iauS06 ( tt1, tt2, x, y );
   
   Pxys ( "X, Y, s using iauXy06 and iauS06", x, y, s );
   Pxys ( "X+DX, Y+DY, s", x+dx, y+dy, s );
   
/* 
   Step 2: Incorporating the IERS corrections dX, dY calculate 
   the matrix c, the GCRS to CIRS matrix.
*/
   iauC2ixys ( x+dx, y+dy, s, c );
   Pmat ( "GCRS TO CIRS matrix using iauC2ixys", c );
   
/*
   Step 3: - Earth rotation angle (function of UT1) that rotates
   from CIRS to TIRS and the equation of the origins
*/
   era = iauEra00 ( ut1a, ut1b );
   printf ( "\nEarth rotation angle = %17.12f degrees", era/DD2R ) ; 
   
   eo = iauEo06a ( tt1, tt2 );
   printf ( "\n(Equation of origins = %17.12f mins time)\n", (eo*1440e0)/(DD2R*15e0) ) ;
   printf ( "+ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -+\n");

/* Step 4 - s' TIO Locator & polar motion (wobble), rotation from 
   TIRS to ITRS 
*/
   sp = iauSp00 ( tt1, tt2 );
   iauPom00 ( xp, yp, sp, w );
   Pmat ( "\nTIRS TO ITRS matrix using iauSp00 and iauPom00 (polar motion)", w );
   
/* Step 5: - Transformation from CIRS (matrix c) to ITRS */
   iauC2tcio ( c, era, w, q1 );
   Pmat ( "\nGCRS TO ITRS matrix using iauC2tcio", q1 );
   printf ( "+ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -+\n");

/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */

   printf ( "+ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -+\n");
   printf ( "Produced by Catherine Hohenkerk, Chair, IAU SOFA Board, 2013 September\n");   
   printf ( "+ - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -+\n");
   
   return (0);
}

void Pmat ( char text[], double r[3][3] )
/* Print matrix R */
{
   printf ( "%s\n", text );
   printf ( "     %19.15f %19.15f %19.15f\n", r[0][0], r[0][1], r[0][2] );
   printf ( "     %19.15f %19.15f %19.15f\n", r[1][0], r[1][1], r[1][2] );
   printf ( "     %19.15f %19.15f %19.15f\n", r[2][0], r[2][1], r[2][2] );
    
   return;
}

void Pxys ( char text[], double x, double y, double s )
/* Print X, Y, s in arc seconds  */
{      
   printf ( "%s\n", text );
   printf ( "   X (\") = %13.7f\n", x/DAS2R );
   printf ( "   Y (\") = %13.7f\n", y/DAS2R );
   printf ( "   s (\") = %13.7f\n", s/DAS2R );

   printf ("\n");
   
   return;
}

void repjd ( char s[], double t1, double t2 )
{
/* Print Julian date  */
   printf ( " %8s", s);
   printf ("%.1f + %.10f = %.10f\n", t1, t2, t1+t2);
   
   return;
}
